SuperStrict

Import BaH.libsvg


' A small example showing :
'   Load an SVG
'   Draw it in a cairo context at different scales and tanslations

' Scale the balls using Q, W, A and Z
' Move the balls using Arrow keys.



Local balls:TBalls = TBalls.create()


Graphics 256,256,0
	
SetBlend ALPHABLEND
	
SetClsColor 255,255,255
	
Repeat
	
	Cls
	
	balls.paint()

	Flip

Until KeyHit(key_escape)
	
End

Type TBalls

	Const width:Int = 256
	Const height:Int = 256

	Field scaleChange:Float = 0.01
	Field offsetChange:Float = 1
	Field xOffset:Float = 0
	Field yOffset:Float = 0
	Field xScale:Float = 1.0
	Field yScale:Float = 1.0

	Field cairo:TSvgCairo = TSvgCairo.Create(TCairoImageSurface.CreateForPixmap(width, height,PF_BGRA8888))


	Function Create:TBalls()
		Local this:TBalls = New TBalls
		
		this.loadSVG()

		Return this
	End Function


	Method loadSVG()
		cairo.ParseSVG("balls.svg")
	End Method

	Method Paint()
	
		
		' define a clipped drawing area
		cairo.rectangle(0, 0, width, height)
		cairo.clip()

		' clear the background (white)
		cairo.Clear(1, 1, 1, width, height)
		
		update()
		draw()
	
		
		Local image:TImage = LoadImage(TCairoImageSurface(cairo.getTarget()).pixmap())
		DrawImage image, 0, 0
		
	End Method

	Method update()
		If KeyDown(KEY_Q) Then
			xScale:- scaleChange
		End If
		If KeyDown(KEY_W) Then
			xScale:+ scaleChange
		End If
		If KeyDown(KEY_A) Then
			yScale:+ scaleChange
		End If
		If KeyDown(KEY_Z) Then
			yScale:- scaleChange
		End If
		If KeyDown(KEY_LEFT) Then
			xOffset:- offsetChange
		End If
		If KeyDown(KEY_RIGHT) Then
			xOffset:+ offsetChange
		End If
		If KeyDown(KEY_UP) Then
			yOffset:- offsetChange
		End If
		If KeyDown(KEY_DOWN) Then
			yOffset:+ offsetChange
		End If
	End Method

	Method draw()
		
		cairo.Save()
	
		cairo.Scale(xScale, yScale)
		cairo.Translate(xOffset, yOffset)

		cairo.RenderSVG()

		cairo.Restore()

	End Method

End Type