SuperStrict

Rem
bbdoc: Cairo - libSVG
End Rem
Module BaH.libsvg

ModuleInfo "Version: 1.02"
ModuleInfo "License: LGPL"
ModuleInfo "Author: University of Southern California and Carl D. Worth"
ModuleInfo "Credit: Adapted for BlitzMax by Bruce Henderson"
ModuleInfo "Modserver: BRL"

ModuleInfo "History: 1.02"
ModuleInfo "History: Updates to cvs versions of libsvg and libsvg-cairo (1 Dec 2006)"
ModuleInfo "History: 1.01"
ModuleInfo "History: Changed to support revised Type structure for TCairo"
ModuleInfo "History: 1.00 Initial Release (libsvg 0.1.4 and libsvg-cairo 0.1.6)"

' changes to svg.c
' Removed libgen.h include, and modified svg_parse to accept dirname string instead of using glib dirname() function.
' ...no longer reliant on glib, as we pass in a dir for it...

' changes to svg_cairo.c
' Added svg_cairo_init() function as I couldn't work out how to handle Byte Ptr Ptr... :-p

' changes to svg_parser_libxml.c
' Changed use of xmlFree to xmlMemFree.

Import BaH.Cairo
Import BaH.Libxml
Import Pub.zlib
Import Pub.libjpeg
Import Pub.libpng
Import BRL.Filesystem

Import "../cairo.mod/src/*.h"
Import "../libxml.mod/src/*.h"
Import "../../pub.mod/zlib.mod/*.h"
Import "../../pub.mod/libjpeg.mod/*.h"
Import "../../pub.mod/libpng.mod/*.h"

' headers
Import "libsvg/*.h"
Import "libsvg-cairo/*.h"

' libsvg
Import "libsvg/svg.c"
Import "libsvg/svg_ascii.c"
Import "libsvg/svg_attribute.c"
Import "libsvg/svg_color.c"
Import "libsvg/svg_element.c"
Import "libsvg/svg_gradient.c"
Import "libsvg/svg_group.c"
Import "libsvg/svg_hash.c"
Import "libsvg/svg_image.c"
Import "libsvg/svg_length.c"
Import "libsvg/svg_paint.c"
Import "libsvg/svg_parser.c"
Import "libsvg/svg_parser_libxml.c"
Import "libsvg/svg_path.c"
Import "libsvg/svg_pattern.c"
Import "libsvg/svg_str.c"
Import "libsvg/svg_style.c"
Import "libsvg/svg_text.c"
Import "libsvg/svg_transform.c"


' libsvg-cairo
Import "libsvg-cairo/svg_cairo.c"
Import "libsvg-cairo/svg_cairo_state.c"

Extern
	Function svg_cairo_init:Byte Ptr()
	Function svg_cairo_destroy:Int(svgCairoPtr:Byte Ptr)
	Function svg_cairo_parse:Int(svgCairoPtr:Byte Ptr, filename:Byte Ptr, dirname:Byte Ptr)
	Function svg_cairo_get_size(svgCairoPtr:Byte Ptr, width:Int Var, height:Int Var)
	Function svg_cairo_render(svgCairoPter:Byte Ptr, cairoPtr:Byte Ptr)
	Function svg_cairo_set_viewport_dimension(svgCairoPtr:Byte Ptr, width:Int, height:Int)
End Extern

Const SVG_CAIRO_STATUS_SUCCESS:Int = 0
Const SVG_CAIRO_STATUS_NO_MEMORY:Int = 1
Const SVG_CAIRO_STATUS_IO_ERROR:Int = 2
Const SVG_CAIRO_STATUS_FILE_NOT_FOUND:Int = 3
Const SVG_CAIRO_STATUS_INVALID_VALUE:Int = 4
Const SVG_CAIRO_STATUS_INVALID_CALL:Int = 5
Const SVG_CAIRO_STATUS_PARSE_ERROR:Int = 6
Const SVG_CAIRO_STATUS_INIT_ERROR:Int = 7

Rem
bbdoc: SVG savvy Cairo Type.
about: This Type introduces some methods which allow the loading and rendering of SVG files into
a Cairo context.
End Rem
Type TSVGCairo Extends TCairo

	' A pointer to the underlying data.
	Field svgContextPtr:Byte Ptr
	
	Function create:TSVGCairo(surf:TCairoSurface)
		Local cr:TSVGCairo = New TSVGCairo
	
		cr.surface = surf
		
		cr.createContext()
		
		Return cr
	End Function
	

	Method NoSVGContextError()
		If svgContextPtr = Null Then
			Throw New TSvgCairoNoSVGContextException
		End If
	End Method

	Rem
	bbdoc: Loads and parses a .svg file ready for rendering with Cairo.
	returns: 0 for success, or an error status value.
	about: 
	<p>
	The following details possible status values :
	</p>
	<table>
	<tr><th>Constant</th></tr>
	<tr><td>SVG_CAIRO_STATUS_SUCCESS</td></tr>
	<tr><td>SVG_CAIRO_STATUS_NO_MEMORY</td></tr>
	<tr><td>SVG_CAIRO_STATUS_IO_ERROR</td></tr>
	<tr><td>SVG_CAIRO_STATUS_FILE_NOT_FOUND</td></tr>
	<tr><td>SVG_CAIRO_STATUS_INVALID_VALUE</td></tr>
	<tr><td>SVG_CAIRO_STATUS_INVALID_CALL</td></tr>
	<tr><td>SVG_CAIRO_STATUS_PARSE_ERROR</td></tr>
	<tr><td>SVG_CAIRO_STATUS_INIT_ERROR</td></tr>
	</table>
	End Rem
	Method ParseSVG:Int(filename:String)

		Local ret:Int
	
		If filename = Null Or filename = "" Then
			ret = SVG_CAIRO_STATUS_FILE_NOT_FOUND
		Else

			svgContextPtr = svg_cairo_init()
	
			If svgContextPtr <> Null Then
				Local dir:String = ExtractDir(filename)
				If dir = "<bad_dir>" Then
					dir = "."
				End If
				ret = svg_cairo_parse(svgContextPtr, filename.toCString(), dir.toCString())
			Else
				ret = SVG_CAIRO_STATUS_INIT_ERROR
			End If

		End If

		Return ret
	End Method

	Rem
	bbdoc: Returns the current width and height of the current SVG image into the specified parameters.
	returns: 0 for success, or an error status value. (See #ParseSVG for list of possible values)
	about:Parameters:
	<ul>
	<li><b>width</b> : width of the current SVG image <i>(out parameter)</i></li>
	<li><b>height</b> : height of the current SVG image <i>(out parameter)</i></li>
	</ul>
	End Rem
	Method GetSVGSize:Int(width:Int Var, height:Int Var)
		NoSVGContextError()
		Return svg_cairo_get_size(svgContextPtr, width, height)
	End Method

	Rem
	bbdoc: Renders the current SVG image onto the Cairo context.
	returns: 0 for success, or an error status value. (See #ParseSVG for list of possible values)
	about: How the image renders is dependent on use of methods such as
	<a href="../../cairo.mod/doc/commands.html#Scale">Scale</a>,
	<a href="../../cairo.mod/doc/commands.html#Translate">Translate</a>, 
	<a href="../../cairo.mod/doc/commands.html#Rotate">Rotate</a> and
	<a href="../../cairo.mod/doc/commands.html#PaintWithAlpha">PaintWithAlpha</a>.
	End Rem
	Method RenderSVG:Int()
		NoSVGContextError()
		Return svg_cairo_render(svgContextPtr, contextPtr)
	End Method

	Rem
	bbdoc: Removes the current SVG context and its associated resources.
	returns: 0 for success, or an error status value. (See #ParseSVG for list of possible values)
	End Rem
	Method DestroySVG:Int()
		NoSVGContextError()
		Return svg_cairo_destroy(svgContextPtr)
	End Method

End Type

Type TSvgCairoNoSVGContextException
	Method toString:String()
		Return "Call ParseSVG() before attempting to use the SVG methods"
	End Method
End Type